@@ -3,8 +3,9 @@
 import csv
 import json
 from collections import OrderedDict
-from os.path import join
-from typing import Any, Dict
+from collections.abc import Callable, Iterable, Mapping, Sequence
+from pathlib import Path
+from typing import Any
 
 from ruamel.yaml import (
     YAML,
@@ -13,8 +14,8 @@ from ruamel.yaml import (
     add_representer,
 )
 
-from .matching import match_template_variables
-from .typehint import ListTuple, ListTupleDict
+from hdx.utilities.frictionless_wrapper import get_frictionless_tableresource
+from hdx.utilities.matching import match_template_variables
 
 
 class UnPrettyRTRepresenter(RoundTripRepresenter):
@@ -49,13 +50,13 @@ representers = {
 }
 
 
-def save_text(string: str, path: str, encoding: str = "utf-8") -> None:
+def save_text(string: str, path: Path | str, encoding: str = "utf-8") -> None:
     """Save text string to file.
 
     Args:
-        string (str): String to save
-        path (str): Path to file
-        encoding (str): Encoding of file. Defaults to utf-8.
+        string: String to save
+        path: Path to file
+        encoding: Encoding of file. Defaults to utf-8.
 
     Returns:
         None
@@ -66,7 +67,7 @@ def save_text(string: str, path: str, encoding: str = "utf-8") -> None:
 
 def save_yaml(
     object: Any,
-    path: str,
+    path: Path | str,
     encoding: str = "utf-8",
     pretty: bool = False,
     sortkeys: bool = False,
@@ -74,11 +75,11 @@ def save_yaml(
     """Save dictionary to YAML file preserving order if it is an OrderedDict.
 
     Args:
-        object (Any): Python object to save
-        path (str): Path to YAML file
-        encoding (str): Encoding of file. Defaults to utf-8.
-        pretty (bool): Whether to pretty print. Defaults to False.
-        sortkeys (bool): Whether to sort dictionary keys. Defaults to False.
+        object: Python object to save
+        path: Path to YAML file
+        encoding: Encoding of file. Defaults to utf-8.
+        pretty: Whether to pretty print. Defaults to False.
+        sortkeys: Whether to sort dictionary keys. Defaults to False.
 
     Returns:
         None
@@ -100,7 +101,7 @@ def save_yaml(
 
 def save_json(
     object: Any,
-    path: str,
+    path: Path | str,
     encoding: str = "utf-8",
     pretty: bool = False,
     sortkeys: bool = False,
@@ -108,11 +109,11 @@ def save_json(
     """Save dictionary to JSON file preserving order if it is an OrderedDict.
 
     Args:
-        object (Any): Python object to save
-        path (str): Path to JSON file
-        encoding (str): Encoding of file. Defaults to utf-8.
-        pretty (bool): Whether to pretty print. Defaults to False.
-        sortkeys (bool): Whether to sort dictionary keys. Defaults to False.
+        object: Python object to save
+        path: Path to JSON file
+        encoding: Encoding of file. Defaults to utf-8.
+        pretty: Whether to pretty print. Defaults to False.
+        sortkeys: Whether to sort dictionary keys. Defaults to False.
 
     Returns:
         None
@@ -134,11 +135,11 @@ def save_json(
 
 
 def save_hxlated_output(
-    configuration: Dict,
-    rows: ListTuple[ListTupleDict],
+    configuration: dict,
+    rows: Sequence[Sequence | Mapping],
     includes_header: bool = True,
     includes_hxltags: bool = False,
-    output_dir: str = "",
+    output_dir: Path | str = "",
     **kwargs: Any,
 ) -> None:
     """Save rows with header and HXL hashtags. Currently, JSON and/or csv
@@ -149,16 +150,17 @@ def save_hxlated_output(
     variables needed by the metadata defined in the configuration.
 
     Args:
-        configuration (Dict): Configuration for input and output
-        rows (ListTuple[ListTupleDict]): Rows of data
-        includes_header (bool): Whether rows includes header. Defaults to True,
-        includes_hxltags (bool): Whether rows includes HXL hashtags. Defaults to False.
-        output_dir (str): Output directory. Defaults to "".
+        configuration: Configuration for input and output
+        rows: Rows of data
+        includes_header: Whether rows includes header. Defaults to True,
+        includes_hxltags: Whether rows includes HXL hashtags. Defaults to False.
+        output_dir: Output directory. Defaults to "".
         **kwargs: Variables to use when evaluating template arguments
 
     Returns:
         None
     """
+    output_dir = Path(output_dir)
     row0 = rows[0]
     if includes_header:
         if isinstance(row0, dict):
@@ -189,8 +191,7 @@ def save_hxlated_output(
     if csv_configuration:
         csv_hxltags = csv_configuration.get("hxltags", hxltags)
         csv_headers = [hxltag_to_header[hxltag] for hxltag in csv_hxltags]
-        csv_file = open(
-            join(output_dir, csv_configuration["filename"]),
+        csv_file = (output_dir / csv_configuration["filename"]).open(
             "w",
             encoding="utf-8",
             newline="\n",
@@ -220,7 +221,7 @@ def save_hxlated_output(
         else:
             metadata_json = None
 
-        output_json = open(join(output_dir, json_configuration["filename"]), "w")
+        output_json = (output_dir / json_configuration["filename"]).open("w")
 
         if metadata_json:
             metadata_key = metadata_configuration.get("key", "metadata")
@@ -268,3 +269,114 @@ def save_hxlated_output(
         output_json.close()
     if csv_file:
         csv_file.close()
+
+
+def save_iterable(
+    filepath: Path | str,
+    rows: Iterable[Sequence | Mapping],
+    headers: int | Sequence[str] | None = None,
+    columns: Sequence[int] | Sequence[str] | None = None,
+    format: str = "csv",
+    encoding: str | None = None,
+    row_function: Callable[[dict], dict | None] | None = None,
+) -> list:
+    """Save an iterable of rows in dict or list form to a csv. (The headers
+    argument is either a row number (rows start counting at 1), or the actual
+    headers defined as a list of strings. If not set, all rows will be treated
+    as containing values.)
+
+    Args:
+        filepath: Path to write to
+        rows: List of rows in dict or list form
+        headers: Headers to write. Defaults to None.
+        columns: Columns to write. Defaults to all.
+        format: Format to write. Defaults to csv.
+        encoding: Encoding to use. Defaults to None (infer encoding).
+        row_function: Row function to call for each row. Defaults to None.
+
+    Returns:
+        List of rows written to file
+    """
+    if row_function is None:
+
+        def row_function(row):
+            return row
+
+    newrows = []
+    rows = iter(rows)
+    try:
+        row = next(rows)
+    except StopIteration:
+        return newrows
+    if isinstance(row, dict):
+        has_header = True
+        row = row_function(row)
+        if columns:
+            if row is not None:
+                newrow = {}
+                for column in columns:
+                    if column in row:
+                        newrow[column] = row[column]
+                newrows.append(newrow)
+            for row in rows:
+                row = row_function(row)
+                if row is None:
+                    continue
+                newrow = {}
+                for column in columns:
+                    if column in row:
+                        newrow[column] = row[column]
+                newrows.append(newrow)
+            if headers is None:
+                headers = columns
+        else:
+            if row is not None:
+                newrows.append(row)
+            for row in rows:
+                row = row_function(row)
+                if row is None:
+                    continue
+                newrows.append(row)
+    else:
+        if headers is None:
+            headers = 1
+        if isinstance(headers, int):
+            headers_rowno = headers
+            headers = row
+            for i in range(headers_rowno - 1):
+                headers = next(rows)
+        else:
+            headers_rowno = None
+        has_header = False
+        row = row_function(row)
+        if columns:
+            if headers_rowno is None and row is not None:
+                newrow = []
+                for column in columns:
+                    newrow.append(row[column - 1])
+                newrows.append(newrow)
+            for row in rows:
+                row = row_function(row)
+                if row is None:
+                    continue
+                newrow = []
+                for column in columns:
+                    newrow.append(row[column - 1])
+                newrows.append(newrow)
+        else:
+            if headers_rowno is None and row is not None:
+                newrows.append(row)
+            for row in rows:
+                row = row_function(row)
+                if row is None:
+                    continue
+                newrows.append(row)
+    resource = get_frictionless_tableresource(
+        data=newrows,
+        has_header=has_header,
+        headers=headers,
+        encoding=encoding,
+    )
+    resource.write(filepath, format=format, encoding=encoding)
+    resource.close()
+    return newrows
