@@ -1,7 +1,5 @@
 """Compare Utility Tests"""
 
-from os.path import join
-
 import pytest
 
 from hdx.utilities.compare import assert_files_same, compare_files
@@ -10,11 +8,11 @@ from hdx.utilities.compare import assert_files_same, compare_files
 class TestCompare:
     @pytest.fixture(scope="class")
     def testfile1(self, fixturesfolder):
-        return join(fixturesfolder, "compare", "test_csv_processing.csv")
+        return fixturesfolder / "compare" / "test_csv_processing.csv"
 
     @pytest.fixture(scope="class")
     def testfile2(self, fixturesfolder):
-        return join(fixturesfolder, "compare", "test_csv_processing2.csv")
+        return fixturesfolder / "compare" / "test_csv_processing2.csv"
 
     def test_compare_files(self, testfile1, testfile2):
         result = compare_files(testfile1, testfile2, encoding="utf-8")
